'Utility functions to simplify the use of FreeImage in XBasic programs.
'Written for FreeImage version 3.6.1, and xfree.dll
'written by Ken Minogue, 29 Mar 2005
'modified to be used with XBLite
'
' Programs that import xfi may also need to import xfree, to gain access to some constants
'  such as the FREE_IMAGE_FORMAT and FREE_IMAGE_FILTER constants.
'
' ####################
' #####  PROLOG  #####
' ####################
'
PROGRAM	"xfi"
VERSION	"3.6.1"

IMPORT	"xst"
IMPORT  "xsx"
IMPORT  "kernel32"
IMPORT  "xfree"

TYPE BUFFER
	ULONG	.address
	ULONG	.length
	ULONG	.offset
END TYPE

EXPORT
DECLARE	FUNCTION  Xfi ()
DECLARE FUNCTION  XfiConvert (fif, UBYTE rawData[], UBYTE image[])
DECLARE FUNCTION  XfiLoadFile (fif, file$, UBYTE image[])
DECLARE FUNCTION  XfiLoadImage (file$, UBYTE image[])
DECLARE FUNCTION  XfiLoadResource (fif, name$, type$, module$, UBYTE image[])
DECLARE FUNCTION  XfiOutputMessage$ ()
DECLARE FUNCTION  XfiRescale (UBYTE imageIn[], UBYTE imageOut[], width, height, filter)
DECLARE FUNCTION  XfiSaveFile (fif, file$, UBYTE image[], flag)
DECLARE FUNCTION  XfiSetDebug (debug)
DECLARE FUNCTION  XfiVersion (xfiVersion$, fiVersion$, copyright$)
DECLARE FUNCTION  Blowback ()
END EXPORT

INTERNAL FUNCTION  LoadArrayResource (fif, name$, type$, module$, dataType, ANY)
INTERNAL FUNCTION  LoadBitmapResource (name$, module$, UBYTE image[])
INTERNAL FUNCTION  OpenHandle (UBYTE array[])
INTERNAL FUNCTION  CloseHandle (handle&&)
INTERNAL FUNCTION  OpenMemory (UBYTE image[], dib)
DECLARE  FUNCTION  ReadMemory (bufferAddr&&, size&&, count&&, handle&&)
DECLARE  FUNCTION  WriteMemory (bufferAddr&&, size&&, count&&, handle&&)
DECLARE  FUNCTION  SeekMemory (handle&&, offset, origin)
DECLARE  FUNCTION  TellMemory (handle&&)
DECLARE CFUNCTION  OutputMessage (fif, msg)

'the following functions are in kernel32.dll; the declarations can be moved to
' kernel32.dec if desired
'EXTERNAL FUNCTION  FindResourceA (hModule, lpName, lpType)
'EXTERNAL FUNCTION  SizeofResource (hModule, hResInfo)

'
'
' ####################
' #####  Xfi ()  #####
' ####################
'
' Initialization function, called automatically when xfi.dll is loaded
'
FUNCTION  Xfi ()
	SHARED FreeImageIO io

	STATIC entered
	IF entered THEN RETURN
	entered = $$TRUE

'set up FreeImageIO structure for use by Load/SaveFromHandle functions
	io.read_proc 	= &ReadMemory()
	io.write_proc = &WriteMemory()
	io.seek_proc 	= &SeekMemory()
	io.tell_proc 	= &TellMemory()

	FreeImage_Initialise(0)
	FreeImage_SetOutputMessage (&OutputMessage())
END FUNCTION
'
'
' ###########################
' #####  XfiConvert ()  #####
' ###########################
'
' Converts an input array containing an image in any format supported for
' reading by FreeImage into an XBasic-compatible image[] array. Use instead of
' XfiLoadFile(), when the image comes from a source other than a file.
'
' fif is a FREE_IMAGE_FORMAT code (see xfree.dec). Use fif = -1 ($$FIF_UNKNOWN) to have
'  FreeImage automatically determine the format from the file header or extension - this
'  may not work for all formats, however.
' rawData[] is the image data in a supported format
' image[] is the returned XBasic-compatible image
'
FUNCTION  XfiConvert (fif, UBYTE rawData[], UBYTE image[])
	SHARED FreeImageIO io
	SHARED dib

	DIM image[]
	IFZ io.write_proc THEN RETURN $$TRUE	'not initialized

	error = $$TRUE

	hRawData = OpenHandle (@rawData[])
	IFZ hRawData GOTO Exit

	IF fif == $$FIF_UNKNOWN THEN
		fif = FreeImage_GetFileTypeFromHandle (&io, hRawData, 0)
		IF fif == $$FIF_UNKNOWN GOTO Exit
	END IF
	IFF FreeImage_FIFSupportsReading(fif) GOTO Exit

	dib = FreeImage_LoadFromHandle (fif, &io, hRawData, 0)
	IFZ dib GOTO Exit

	hBmp = OpenMemory (@image[], dib)
	IFZ hBmp GOTO Exit
	FreeImage_SaveToHandle ($$FIF_BMP, dib, &io, hBmp, 0)

	error = $$FALSE

Exit:
	CloseHandle (hRawData)
	CloseHandle (hBmp)
	FreeImage_Unload (dib)
	dib = 0

	RETURN error
END FUNCTION
'
'
' ############################
' #####  XfiLoadFile ()  #####
' ############################
'
' Loads a file containing image data in any format supported for
' reading by FreeImage into an XBasic-compatible image[] array.
'
' fif is a FREE_IMAGE_FORMAT code (see xfree.dec). Use fif = -1 ($$FIF_UNKNOWN) to have
'  FreeImage automatically determine the format from the file header or extension - this
'  may not work for all formats, however.
' file$ is the name of the file containing image data
' image[] is the returned XBasic-compatible image
'
FUNCTION  XfiLoadFile (fif, file$, UBYTE image[])
	SHARED FreeImageIO io
	SHARED dib

	DIM image[]
	IFZ io.write_proc THEN RETURN $$TRUE	'not initialized
	error = $$TRUE

	IF fif == $$FIF_UNKNOWN THEN
		fif = FreeImage_GetFileType (&file$, 0)
		IF fif == $$FIF_UNKNOWN THEN
			fif = FreeImage_GetFIFFromFilename(&file$)
			IF fif == $$FIF_UNKNOWN GOTO Exit
		END IF
	END IF
	IFF FreeImage_FIFSupportsReading(fif) GOTO Exit

	dib = FreeImage_Load (fif, &file$, 0)
	IFZ dib THEN GOTO Exit

	handle = OpenMemory (@image[], dib)

	IFZ handle GOTO Exit
	FreeImage_SaveToHandle ($$FIF_BMP, dib, &io, handle, 0)

	error = $$FALSE

Exit:
	CloseHandle (handle)
	FreeImage_Unload (dib)
	dib = 0
	RETURN error
END FUNCTION
'
'
' ############################
' #####  XfiLoadImage ()  ####
' ############################
'
' This function provides a image loading function with
'  the same arguments as XgrLoadImage().
'
FUNCTION  XfiLoadImage (file$, UBYTE image[])
	RETURN XfiLoadFile (-1, file$, @image[])
END FUNCTION
'
'
' ################################
' #####  XfiLoadResource ()  #####
' ################################
'
' Loads an image from a resource and converts to an XBasic-compatible image[].
'
' fif is a FREE_IMAGE_FORMAT code (see xfree.dec). Use fif = -1 ($$FIF_UNKNOWN) to have
'  FreeImage automatically determine the format from the file header or extension - this
'  may not work for all formats, however.
' name$ is the name given to the resource in the .rc file
' type$ is the resource type, as given in the .rc (eg., "JPEG")
' module$ is the name of the module (EXE or DLL) containing the resource.
'  Use module$ = "" if the resource is in the calling process.
' image[] is the returned XBasic-compatible image
'
FUNCTION  XfiLoadResource (fif, name$, type$, module$, UBYTE image[])
	SHARED FreeImageIO io
	SHARED dib
	UBYTE rawData[]

	DIM image[]
	IFZ io.write_proc THEN RETURN $$TRUE	'not initialized

	SELECT CASE UCASE$(type$)
		CASE "BITMAP"
			IF LoadBitmapResource (name$, module$, @rawData[]) THEN RETURN $$TRUE
		CASE ELSE
			IF LoadArrayResource (fif, name$, type$, module$, $$UBYTE, @rawData[]) THEN RETURN $$TRUE
	END SELECT

	error = $$TRUE

	hRawData = OpenHandle (@rawData[])
	IFZ hRawData GOTO Exit

	IF fif == $$FIF_UNKNOWN THEN
		fif = FreeImage_GetFileTypeFromHandle (&io, hRawData, 0)
		IF fif == $$FIF_UNKNOWN GOTO Exit
	END IF
	IFF FreeImage_FIFSupportsReading(fif) GOTO Exit
	dib = FreeImage_LoadFromHandle (fif, &io, hRawData, 0)
	DIM rawData[]
	IFZ dib GOTO Exit

	hBmp = OpenMemory (@image[], dib)
	IFZ hBmp GOTO Exit
	FreeImage_SaveToHandle ($$FIF_BMP, dib, &io, hBmp, 0)

	error = $$FALSE

Exit:
	CloseHandle (hRawData)
	CloseHandle (hBmp)
	FreeImage_Unload (dib)
	dib = 0
	RETURN error
END FUNCTION
'
'
' ##################################
' #####  XfiOutputMessage$ ()  #####
' ##################################
'
' Return the most recent output message, as set by a FreeImage call to OutputMessage()
' Also clears the output message.
'
' This function will work only if the calling program has not changed the
'  output message function by calling FreeImage_SetOutputMessage()
'
FUNCTION XfiOutputMessage$ ()
	SHARED outputMessage$

	m$ = outputMessage$
	outputMessage$ = ""
	RETURN m$
END FUNCTION
'
'
' ###########################
' #####  XfiRescale ()  #####
' ###########################
'
' Creates a new image by rescaling the input image to specified dimensions
'
' imageIn[] is the original image data in XBasic-compatible format (BMP)
' imageOut[] is the rescaled image, XBasic-compatible
' width and height are the required dimensions for the rescaled image, in pixels
' filter is a FREE_IMAGE_FILTER constant (see xfree.dec)
'
FUNCTION  XfiRescale (UBYTE imageIn[], UBYTE imageOut[], width, height, filter)
	SHARED BUFFER buffer[]
	SHARED FreeImageIO io

	DIM imageOut[]
	IFZ io.write_proc THEN RETURN $$TRUE	'not initialized

	hIn = OpenHandle (@imageIn[])
	IFZ hIn THEN RETURN

	error = $$TRUE

	dibIn = FreeImage_LoadFromHandle ($$FIF_BMP, &io, hIn, 0)
	IFZ dibIn GOTO Exit

	dibOut = FreeImage_Rescale (dibIn, width, height, filter)
	FreeImage_Unload (dibIn)
	IFZ dibOut GOTO Exit 'invalid bit depth or not enough memory

	hOut = OpenMemory (@imageOut[], dibOut)
	IFZ hOut GOTO Exit
	FreeImage_SaveToHandle ($$FIF_BMP, dibOut, &io, hOut, 0)

	error = $$FALSE

Exit:
	CloseHandle (hIn)
	CloseHandle (hOut)
	FreeImage_Unload (dibOut)

	RETURN error
END FUNCTION
'
'
' ############################
' #####  XfiSaveFile ()  #####
' ############################
'
' Saves a XBasic image[] as a file in any format supported by FreeImage for writing.
'	 fif is one of the FREE_IMAGE_FORMAT constants, eg $$FIF_BMP, $$FIF_JPEG (see xfree.dec)
'	 file$ is the name of the file to create
'  image[] contains the XBasic image data
'  flag has varying meaning depending on the format - see
'    FreeImage documentation for FreeImage_Save()
'  for saving gif files, use flag = $$FALSE to save 24-bit images as 256 color gif
'    and flag = $$TRUE to save as 256 color grayscale b&w gif
'
FUNCTION  XfiSaveFile (fif, file$, UBYTE image[], flag)
	SHARED FreeImageIO io
	SHARED dib

	IFZ io.write_proc THEN RETURN($$TRUE)	'not initialized
	IFZ image[] THEN RETURN

	IFF FreeImage_FIFSupportsWriting(fif) THEN RETURN $$TRUE

	error = $$TRUE

	handle = OpenHandle (@image[])
	IFZ handle GOTO Exit
	dib = FreeImage_LoadFromHandle ($$FIF_BMP, &io, handle, 0)
	IFZ dib GOTO Exit

	' convert dib to 8 bit if saving as GIF, result will be black&white
	' otherwise, would need to quantize image and reduce colors to 8-bit
	bpp = FreeImage_GetBPP (dib)
	IF fif = $$FIF_GIF && bpp > 8 THEN

		IFZ flag THEN
			dib24 = FreeImage_ConvertTo24Bits (dib)
			dibq = FreeImage_ColorQuantize (dib24, $$FIQ_NNQUANT)
			ret = FreeImage_Save (fif, dibq, &file$, flag)
			FreeImage_Unload (dib24)
			FreeImage_Unload (dibq)
			IFZ ret THEN GOTO Exit
		ELSE
			dib8bits = FreeImage_ConvertTo8Bits (dib)
			ret = FreeImage_Save (fif, dib8bits, &file$, flag)
			FreeImage_Unload (dib8bits)
			IFZ ret THEN GOTO Exit
		END IF

	ELSE
		IFZ FreeImage_Save (fif, dib, &file$, flag) GOTO Exit
	END IF

	error = $$FALSE

Exit:
	CloseHandle (handle)
	FreeImage_Unload (dib)
	dib = 0

	RETURN error
END FUNCTION
'
'
' ############################
' #####  XfiSetDebug ()  #####
' ############################
'
' causes some debugging messages to be printed on the console, if debug is TRUE
FUNCTION  XfiSetDebug (debug)
	#debug = debug
END FUNCTION
'
'
' ###########################
' #####  XfiVersion ()  #####
' ###########################
'
' returns the Xfi version, and the FreeImage version and copyright strings
'
FUNCTION  XfiVersion (xfiVersion$, fiVersion$, copyright$)
	xfiVersion$ = VERSION$(0)
	fiVersion$ 	= FreeImage_GetVersion ()
	copyright$ 	= FreeImage_GetCopyrightMessage ()
END FUNCTION
'
'
' #########################
' #####  Blowback ()  #####
' #########################
'
FUNCTION  Blowback ()
	SHARED hLib
	SHARED dib

	IF dib THEN FreeImage_Unload (dib)
	dib = 0

	IF hLib THEN
		FreeImage_DeInitialise ()
		FreeLibrary (hLib)
		hLib = 0
	END IF
END FUNCTION
'
'
' ##################################
' #####  LoadArrayResource ()  #####
' ##################################
'
' Loads a resource into an array.
'
' name$ is the name given to the resource in the .rc file
' type$ is the resource type, as given in the .rc (eg., "JPEG")
' module$ is the name of the module (EXE or DLL) containing the resource. Use
'    module$ = "" if the resource is in the calling process.
' dataType is the desired data type of the returned array.
' array[] is the returned array, containing the resource data.
'
FUNCTION  LoadArrayResource (fif, name$, type$, module$, dataType, array[])
'get the instance handle for the executable module (EXE or DLL) containing the resource
	IF module$ THEN
			hInstance = LoadLibraryA (&module$)
		ELSE	'if module$ = "", get handle of the running EXE
			hInstance = GetModuleHandleA (0)
	END IF
	err = GetLastError()
	IFZ hInstance THEN RETURN ($$TRUE)

'find the resource within the module
	hRsrc = FindResourceA (hInstance, &name$, &type$)
	IFZ hRsrc THEN
		IF module$ THEN FreeLibrary (hInstance)
		RETURN ($$TRUE)
	END IF
	rsrcSize = SizeofResource (hInstance, hRsrc)
	lpRsrc = LoadResource (hInstance, hRsrc)

'copy resource data into array[]
	XstGetTypedArray (dataType, rsrcSize, @array[])
	XstCopyMemory (lpRsrc, &array[], rsrcSize)

	IF module$ THEN FreeLibrary (hInstance)
END FUNCTION
'
'
' ###################################
' #####  LoadBitmapResource ()  #####
' ###################################
'
' Loads a BITMAP resource into an XBasic-compatible image[]
'
' name$ is the name given to the BITMAP in the .rc file
' module$ is the name of the module (EXE or DLL) containing the BITMAP. Use
'    module$ = "" if the BITMAP is in the calling process.
' image[] is the returned array, containing the BITMAP data.
'
FUNCTION  LoadBitmapResource (name$, module$, UBYTE image[])
	$RT_BITMAP = 2

'get the instance handle for the executable module (EXE or DLL) containing the resource
	IF module$ THEN
			hInstance = LoadLibraryA (&module$)
		ELSE	'if module$ = "", get handle of the running EXE
			hInstance = GetModuleHandleA (0)
	END IF
	IFZ hInstance THEN RETURN ($$TRUE)

'find the bitmap resource within the module
	hRsrc = FindResourceA (hInstance, &name$, $RT_BITMAP)
	IFZ hRsrc THEN
		IF module$ THEN FreeLibrary (hInstance)
		RETURN ($$TRUE)
	END IF
	rsrcSize = SizeofResource (hInstance, hRsrc)
	lpImage = LoadResource (hInstance, hRsrc)

'copy BITMAPINFOHEADER, color table, and bitmap data into image[]
	DIM image[rsrcSize+13]
	XstCopyMemory (lpImage, &image[14], rsrcSize)

'create a BITMAPFILEHEADER in the first 14 bytes of image[]
	bitCount = USHORTAT(lpImage, 14)	'.biBitCount member of BITMAPINFOHEADER
	SELECT CASE bitCount
		CASE 1	: colorTableEntries = 2
		CASE 4	: colorTableEntries = 16
		CASE 8	: colorTableEntries = 256
		CASE 24	: colorTableEntries = 0
	END SELECT
	colorTableSize = colorTableEntries * 4

	image[0] = 'B': image[1] = 'M'
	XLONGAT(&image[2]) = rsrcSize + 14
	USHORTAT(&image[10]) = 54 + colorTableSize

	IF module$ THEN FreeLibrary (hInstance)
END FUNCTION
'
'
' ###########################
' #####  OpenHandle ()  #####
' ###########################
'
' Assigns a handle (index in buffer[]) to the input array, which must not be empty.
' Call CloseHandle() when the handle is no longer needed.
'
FUNCTION  OpenHandle (UBYTE array[])
	SHARED BUFFER buffer[]

	IFZ array[] THEN RETURN

	IFZ buffer[] THEN DIM buffer[10]
	FOR handle = 1 TO UBOUND(buffer[])
		IFZ buffer[handle].address THEN EXIT FOR	'found empty slot
	NEXT handle
	IF handle > UBOUND(buffer[]) THEN
		REDIM buffer[handle + 10]
	END IF

	buffer[handle].address = &array[]
	buffer[handle].length = SIZE(array[])
	buffer[handle].offset = 0

	RETURN(handle)
END FUNCTION
'
'
' ############################
' #####  CloseHandle ()  #####
' ############################
'
'	makes a memory handle available for re-use
'	note that the associated memory (the array) is not released.
'
FUNCTION  CloseHandle (ULONG handle)
	SHARED BUFFER buffer[]

	IF (handle <= 0) OR (handle > UBOUND(buffer[])) THEN RETURN
	buffer[handle].address = 0
	buffer[handle].length = 0
	buffer[handle].offset = 0
END FUNCTION
'
'
' ###########################
' #####  OpenMemory ()  #####
' ###########################
'
'	allocates memory for a BMP image
'
FUNCTION  OpenMemory (UBYTE image[], dib)
	DIM image[]

	IFZ dib THEN RETURN
	dibSize = FreeImage_GetDIBSize (dib)
	IFZ dibSize THEN RETURN

	imageSize = dibSize + 14 'SIZE(BITMAPFILEHEADER)
	DIM image[imageSize-1]

	RETURN OpenHandle(@image[])
END FUNCTION
'
'
' ###########################
' #####  ReadMemory ()  #####
' ###########################
'
' Function called by FreeImage to read data from a memory block. The read
'  starts at the current offset of the pointer (buffer.offset), and on exit
'  the pointer is set to the next unread byte.
'
'  All input arguments are supplied by FreeImage, except handle, which is sent to FreeImage
'  when one of the FreeImage_LoadXXXFromHandle() functions is called.
'   bufferAddr: address of buffer to receive data
'         size: size (bytes) of each element read from memory
'        count: number of elements to read
'       handle: handle to memory block (created by OpenMemory())
' RETURN value: number of bytes actually read
'
FUNCTION  ReadMemory (ULONG bufferAddr, ULONG size, ULONG count, ULONG handle)
	SHARED BUFFER buffer[]
	ULONG bytes, firstByte, lastByte
	BUFFER buffer

	IF #debug THEN
		PRINT "Enter ReadMemory for handle"; handle;
		PRINT ","; size*count; " bytes to buffer at "; HEXX$(bufferAddr)
	END IF

	IF (handle <= 0) OR (handle > UBOUND(buffer[])) THEN bytes = 0: GOTO ReadExit
	buffer = buffer[handle]
	IFZ buffer.address THEN bytes = 0: GOTO ReadExit

	bytes = size * count
	IF bytes THEN
		firstByte = buffer.address + buffer.offset
		lastByte = buffer.address + buffer.length
		IF firstByte >= lastByte THEN bytes = 0: GOTO ReadExit
		IF ((firstByte + bytes) > lastByte) THEN bytes = lastByte - firstByte
		XstCopyMemory (firstByte, bufferAddr, bytes)
		buffer.offset = buffer.offset + bytes
		IF #debug THEN
			PRINT " ";
			FOR i = 0 TO 15
				IF i >= bytes THEN EXIT FOR
				PRINT HEXX$(UBYTEAT(bufferAddr, i), 2); " ";
			NEXT i
			PRINT
		END IF
	END IF

	buffer[handle] = buffer

ReadExit:
	IF #debug THEN PRINT " Exit ReadMemory: Return"; bytes;" bytes\n"
	RETURN(bytes)
END FUNCTION
'
'
' ############################
' #####  WriteMemory ()  #####
' ############################
'
' Function called by FreeImage to write data to a memory block. The write
'  starts at the current offset of the pointer (buffer.offset), and on exit
'  the pointer is set to the next byte following the written bytes.
'
'  All input arguments are supplied by FreeImage, except handle, which is sent to FreeImage
'  when one of the FreeImage_SaveXXXToHandle() functions is called.
'   bufferAddr: address of buffer containing data to be written
'         size: size (bytes) of each element written to memory
'        count: number of elements to write
'       handle: handle to memory block (created by OpenMemory())
' RETURN value: number of elements actually written
'
FUNCTION  WriteMemory (ULONG bufferAddr, ULONG size, ULONG count, ULONG handle)
	SHARED BUFFER buffer[]
	ULONG bytes, firstByte, lastByte, actualCount
	BUFFER buffer

	IF #debug THEN
		PRINT "Enter WriteMemory for handle"; handle;
		PRINT ","; size*count; " bytes from buffer at "; HEXX$(bufferAddr)
	END IF

	actualCount = 0
	IF (handle <= 0) OR (handle > UBOUND(buffer[])) GOTO ExitWrite
	buffer = buffer[handle]
	IFZ buffer.address GOTO ExitWrite

	bytes = size * count
	IF bytes THEN
		firstByte = buffer.address + buffer.offset
		lastByte = buffer.address + buffer.length
		IF firstByte >= lastByte GOTO ExitWrite
		IF ((firstByte + bytes) > lastByte) THEN bytes = lastByte - firstByte
		actualCount = bytes\size
		bytes = size * actualCount
		XstCopyMemory (bufferAddr, firstByte, bytes)
		buffer.offset = buffer.offset + bytes
		IF #debug THEN
			PRINT " ";
			FOR i = 0 TO 15
				IF i >= bytes THEN EXIT FOR
				PRINT HEXX$(UBYTEAT(firstByte, i), 2); " ";
			NEXT i
			PRINT
		END IF
	END IF

	buffer[handle] = buffer

ExitWrite:
	IF #debug THEN PRINT " Exit WriteMemory: Return"; actualCount;" blocks\n"
	RETURN (actualCount)
END FUNCTION
'
'
' ###########################
' #####  SeekMemory ()  #####
' ###########################
'
'	Function called by FreeImage to set data pointer in memory block.
'  All input arguments are supplied by FreeImage, except handle, which is sent to FreeImage
'  when one of the FreeImage_SaveXXXToHandle() functions is called.
'
'    handle is the base address of the memory block
'    offset is the location to put the pointer
'    origin is the mode
'				$$SEEK_SET means the offset is taken from the beginning of the block
'       $$SEEK_CUR means the offset is taken from the current position of the pointer
'       $$SEEK_END means the offset is taken from the end of the block (not used by FreeImage)
'
FUNCTION  SeekMemory (ULONG handle, offset, origin)
	SHARED BUFFER buffer[]
	BUFFER buffer

	IF #debug THEN
		PRINT "Enter SeekMemory for handle"; handle;
		PRINT ", offset "; offset;
	END IF
	error = -1
	IF (handle <= 0) OR (handle > UBOUND(buffer[])) GOTO ExitSeek
	buffer = buffer[handle]
	IFZ buffer.address GOTO ExitSeek

	SELECT CASE origin
		CASE $$SEEK_SET
			IF #debug THEN PRINT " from beginning";
			position = offset
		CASE $$SEEK_CUR
			IF #debug THEN PRINT " from current position";
			position = buffer.offset + offset
		CASE $$SEEK_END
			IF #debug THEN PRINT " from end";
			position = buffer.length - offset - 1
		CASE ELSE				: GOTO ExitSeek
	END SELECT

	IF position >= buffer.length THEN GOTO ExitSeek
	IF position < 0 THEN GOTO ExitSeek

	buffer[handle].offset = position
	error = 0

ExitSeek:
	IF #debug THEN PRINT "\n Exit SeekMemory: Return error "; error; "\n"
	RETURN (error)
END FUNCTION
'
'
' ###########################
' #####  TellMemory ()  #####
' ###########################
'
' Function called by FreeImage to obtain the current position of
'  the data pointer in a memory block
'
FUNCTION  TellMemory (ULONG handle)
	SHARED BUFFER buffer[]
	BUFFER buffer

	IF #debug THEN PRINT "Enter TellMemory for handle"; handle
	offset = -1
	IF (handle <= 0) OR (handle > UBOUND(buffer[])) GOTO ExitTell
	buffer = buffer[handle]
	IFZ buffer.address THEN GOTO ExitTell

	offset = buffer.offset

ExitTell:
	IF #debug THEN PRINT " Exit TellMemory: Return offset"; offset; "\n"
	RETURN (offset)
END FUNCTION
'
'
' ##############################
' #####  OutputMessage ()  #####
' ##############################
'
' Default function to be called by FreeImage with error messages, etc.
' Use XfiOutputMessage$() to retrieve the most recent output message
'
CFUNCTION  OutputMessage (fif, msg)
	SHARED outputMessage$

	IF msg THEN
		outputMessage$ = CSTRING$(msg)
	ELSE
		outputMessage$ = ""
	END IF

END FUNCTION
END PROGRAM
