'XBasic/XBLite wrapper for FreeImage 3.6.1
'written by Ken Minogue, 29 Mar 2005
'
'FreeImage is available at http://freeimage.sourceforge.net

'Most functions simply pass arguments to the corresponding function
' in FreeImage.dll, and pass back the return values.
'
'For FreeImage functions that return a pointer to a string, the wrapper
' functions return an equivalent XBasic string.
'
'For FreeImage functions that return a pointer to a structure, the wrapper
' functions return an equivalent XBasic composite type variable. An
' exception is FreeImage_GetInfo(), which returns a pointer to a BITMAPINFO
' structure.  For explanation see Xfree_GetInfo().

PROGRAM "xfree"
VERSION "3.6.1"

IMPORT "gdi32.dec"
IMPORT "user32.dec"
IMPORT "kernel32" 'for LoadLibraryA(), GetProcAddress(), and RtlMoveMemory()

EXPORT

$$SEEK_SET = 0
$$SEEK_CUR = 1
$$SEEK_END = 2

' Indexes for byte arrays, masks and shifts for treating pixels as words
' These coincide with the order of RGBQUAD and RGBTRIPLE

$$FI_RGBA_RED         = 2
$$FI_RGBA_GREEN       = 1
$$FI_RGBA_BLUE        = 0
$$FI_RGBA_ALPHA       = 3
$$FI_RGBA_RED_MASK    = 0x00FF0000
$$FI_RGBA_GREEN_MASK  = 0x0000FF00
$$FI_RGBA_BLUE_MASK   = 0x000000FF
$$FI_RGBA_ALPHA_MASK  = 0xFF000000
$$FI_RGBA_RED_SHIFT   = 16
$$FI_RGBA_GREEN_SHIFT = 8
$$FI_RGBA_BLUE_SHIFT  = 0
$$FI_RGBA_ALPHA_SHIFT = 24
$$FI_RGBA_RGB_MASK    = 0x00FFFFFF 'FI_RGBA_RED_MASK|FI_RGBA_GREEN_MASK|FI_RGBA_BLUE_MASK

' The 16bit macros only include masks and shifts, since each color
' element is not byte aligned

$$FI16_555_RED_MASK     = 0x7C00
$$FI16_555_GREEN_MASK   = 0x03E0
$$FI16_555_BLUE_MASK    = 0x001F
$$FI16_555_RED_SHIFT    = 10
$$FI16_555_GREEN_SHIFT  = 5
$$FI16_555_BLUE_SHIFT   = 0
$$FI16_565_RED_MASK     = 0xF800
$$FI16_565_GREEN_MASK   = 0x07E0
$$FI16_565_BLUE_MASK    = 0x001F
$$FI16_565_RED_SHIFT    = 11
$$FI16_565_GREEN_SHIFT  = 5
$$FI16_565_BLUE_SHIFT   = 0

' ICC profile support

$$FIICC_DEFAULT       = 0x00
$$FIICC_COLOR_IS_CMYK = 0x01

TYPE FIICCPROFILE
	USHORT  .flags  ' info flag
	ULONG   .size   ' profile's size measured in bytes
	XLONG   .data   ' points to a block of contiguous memory containing the profile
END TYPE

' I/O image format identifiers.
'FREE_IMAGE_FORMAT
$$FIF_UNKNOWN = -1
$$FIF_BMP     = 0
$$FIF_ICO     = 1
$$FIF_JPEG    = 2
$$FIF_JNG     = 3
$$FIF_KOALA   = 4
$$FIF_LBM     = 5
$$FIF_IFF     = $$FIF_LBM
$$FIF_MNG     = 6
$$FIF_PBM     = 7
$$FIF_PBMRAW  = 8
$$FIF_PCD     = 9
$$FIF_PCX     = 10
$$FIF_PGM     = 11
$$FIF_PGMRAW  = 12
$$FIF_PNG     = 13
$$FIF_PPM     = 14
$$FIF_PPMRAW  = 15
$$FIF_RAS     = 16
$$FIF_TARGA   = 17
$$FIF_TIFF    = 18
$$FIF_WBMP    = 19
$$FIF_PSD     = 20
$$FIF_CUT     = 21
$$FIF_XBM     = 22
$$FIF_XPM     = 23
$$FIF_DDS     = 24
$$FIF_GIF     = 25

' Image type used in FreeImage.
'FREE_IMAGE_TYPE
$$FIT_UNKNOWN = 0 ' unknown type
$$FIT_BITMAP  = 1 ' standard image      : 1-, 4-, 8-, 16-, 24-, 32-bit
$$FIT_UINT16  = 2 ' array of unsigned short : unsigned 16-bit
$$FIT_INT16   = 3 ' array of short      : signed 16-bit
$$FIT_UINT32  = 4 ' array of unsigned long  : unsigned 32-bit
$$FIT_INT32   = 5 ' array of long     : signed 32-bit
$$FIT_FLOAT   = 6 ' array of float      : 32-bit IEEE floating point
$$FIT_DOUBLE  = 7 ' array of double     : 64-bit IEEE floating point
$$FIT_COMPLEX = 8 ' array of FICOMPLEX    : 2 x 64-bit IEEE floating point

' Data structure for COMPLEX type  (complex number)
TYPE FICOMPLEX
	DOUBLE .r
	DOUBLE .i
END TYPE

' Image color type used in FreeImage.
'FREE_IMAGE_COLOR_TYPE
$$FIC_MINISWHITE = 0    ' min value is white
$$FIC_MINISBLACK = 1    ' min value is black
$$FIC_RGB        = 2    ' RGB color model
$$FIC_PALETTE    = 3    ' color map indexed
$$FIC_RGBALPHA   = 4    ' RGB color model with alpha channel
$$FIC_CMYK       = 5    ' CMYK color model

' Color quantization algorithms.
'Constants used in FreeImage_ColorQuantize.
'FREE_IMAGE_QUANTIZE
$$FIQ_WUQUANT = 0   ' Xiaolin Wu color quantization algorithm
$$FIQ_NNQUANT = 1   ' NeuQuant neural-net quantization algorithm by Anthony Dekker

' Dithering algorithms.
'Constants used FreeImage_Dither.
'FREE_IMAGE_DITHER
$$FID_FS            = 0 ' Floyd & Steinberg error diffusion
$$FID_BAYER4x4      = 1 ' Bayer ordered dispersed dot dithering  (order 2 dithering matrix)
$$FID_BAYER8x8      = 2 ' Bayer ordered dispersed dot dithering  (order 3 dithering matrix)
$$FID_CLUSTER6x6    = 3 ' Ordered clustered dot dithering  (order 3 - 6x6 matrix)
$$FID_CLUSTER8x8    = 4 ' Ordered clustered dot dithering  (order 4 - 8x8 matrix)
$$FID_CLUSTER16x16  = 5 ' Ordered clustered dot dithering  (order 8 - 16x16 matrix)

' Upsampling / downsampling filters.
'Constants used in FreeImage_Rescale.
'FREE_IMAGE_FILTER
$$FILTER_BOX        = 0 ' Box, pulse, Fourier window, 1st order  (constant) b-spline
$$FILTER_BICUBIC    = 1 ' Mitchell & Netravali's two-param cubic filter
$$FILTER_BILINEAR   = 2 ' Bilinear filter
$$FILTER_BSPLINE    = 3 ' 4th order  (cubic) b-spline
$$FILTER_CATMULLROM = 4 ' Catmull-Rom spline, Overhauser spline
$$FILTER_LANCZOS3   = 5 ' Lanczos3 filter

' Color channels.
'Constants used in color manipulation routines.
'FREE_IMAGE_COLOR_CHANNEL
$$FICC_RGB    = 0 ' Use red, green and blue channels
$$FICC_RED    = 1 ' Use red channel
$$FICC_GREEN  = 2 ' Use green channel
$$FICC_BLUE   = 3 ' Use blue channel
$$FICC_ALPHA  = 4 ' Use alpha channel
$$FICC_BLACK  = 5 ' Use black channel
$$FICC_REAL   = 6 ' Complex images: use real part
$$FICC_IMAG   = 7 ' Complex images: use imaginary part
$$FICC_MAG    = 8 ' Complex images: use magnitude
$$FICC_PHASE  = 9 ' Complex images: use phase

' Metadata support
'  Tag data type information  (based on TIFF specifications)
' Note: RATIONALs are the ratio of two 32-bit integer values.
'FREE_IMAGE_MDTYPE
$$FIDT_NOTYPE     = 0   ' placeholder
$$FIDT_BYTE       = 1   ' 8-bit unsigned integer
$$FIDT_ASCII      = 2   ' 8-bit bytes w/ last byte null
$$FIDT_SHORT      = 3   ' 16-bit unsigned integer
$$FIDT_LONG       = 4   ' 32-bit unsigned integer
$$FIDT_RATIONAL   = 5   ' 64-bit unsigned fraction
$$FIDT_SBYTE      = 6   ' 8-bit signed integer
$$FIDT_UNDEFINED  = 7   ' 8-bit untyped data
$$FIDT_SSHORT     = 8   ' 16-bit signed integer
$$FIDT_SLONG      = 9   ' 32-bit signed integer
$$FIDT_SRATIONAL  = 10  ' 64-bit signed fraction
$$FIDT_FLOAT      = 11  ' 32-bit IEEE floating point
$$FIDT_DOUBLE     = 12  ' 64-bit IEEE floating point
$$FIDT_IFD        = 13  ' 32-bit unsigned integer  (offset)

' Metadata models supported by FreeImage
'FREE_IMAGE_MDMODEL
$$FIMD_NODATA         = -1
$$FIMD_COMMENTS       = 0 ' single comment or keywords
$$FIMD_EXIF_MAIN      = 1 ' Exif-TIFF metadata
$$FIMD_EXIF_EXIF      = 2 ' Exif-specific metadata
$$FIMD_EXIF_GPS       = 3 ' Exif GPS metadata
$$FIMD_EXIF_MAKERNOTE = 4 ' Exif maker note metadata
$$FIMD_EXIF_INTEROP   = 5 ' Exif interoperability metadata
$$FIMD_IPTC           = 6 ' IPTC/NAA metadata
$$FIMD_XMP            = 7 ' Abobe XMP metadata
$$FIMD_GEOTIFF        = 8 ' GeoTIFF metadata
$$FIMD_CUSTOM         = 9 ' Used to attach other metadata types to a dib

'  Handle to a metadata model
TYPE FIMETADATA = XLONG

'  Handle to a FreeImage tag
TYPE FITAG = XLONG

' File IO routines
' Prototypes for user-written functions - addresses are used in FREEIMAGEIO
'	EXTERNAL FUNCTION FI_ReadProc (buffer, size, count, handle)
'	EXTERNAL FUNCTION FI_WriteProc (buffer, size, count, handle)
'	EXTERNAL FUNCTION FI_SeekProc (handle, offset, origin)
'	EXTERNAL FUNCTION FI_TellProc (handle)

TYPE FreeImageIO
	FUNCADDR  .read_proc   (XLONG, ULONG, ULONG, XLONG)
	FUNCADDR  .write_proc  (XLONG, ULONG, ULONG, XLONG)
	FUNCADDR  .seek_proc   (XLONG, SLONG, SLONG)
	FUNCADDR  .tell_proc   (XLONG)
END TYPE
TYPE FREEIMAGEIO = FreeImageIO 'older FreeImage versions used all caps

' Plugin routines
' Prototypes for functions in the plugin (addresses of these functions
'  are used in the Plugin TYPE variable):
' EXTERNAL FUNCTION FI_InitProc (Plugin plugin, format_id)
'	EXTERNAL FUNCTION FI_FormatProc ()
'	EXTERNAL FUNCTION FI_DescriptionProc ()
'	EXTERNAL FUNCTION FI_ExtensionListProc ()
'	EXTERNAL FUNCTION FI_RegExprProc ()
'	EXTERNAL FUNCTION FI_OpenProc (FreeImageIO, handle, read)
'	EXTERNAL FUNCTION FI_CloseProc (FreeImageIO, handle, data)
'	EXTERNAL FUNCTION FI_PageCountProc (FreeImageIO, handle, data)
'	EXTERNAL FUNCTION FI_PageCapabilityProc (FreeImageIO, handle, data)
'	EXTERNAL FUNCTION FI_LoadProc (FreeImageIO, handle, page, flags, data)
'	EXTERNAL FUNCTION FI_SaveProc (FreeImageIO, dib, handle, page, flags, data)
'	EXTERNAL FUNCTION FI_ValidateProc (FreeImageIO, handle)
'	EXTERNAL FUNCTION FI_MimeProc ()
'	EXTERNAL FUNCTION FI_SupportsExportBPPProc (bpp)
'	EXTERNAL FUNCTION FI_SupportsExportTypeProc (type)
'	EXTERNAL FUNCTION FI_SupportsICCProfilesProc ()

TYPE Plugin
	FUNCADDR  .format_proc ()
	FUNCADDR  .description_proc ()
	FUNCADDR  .extension_proc ()
	FUNCADDR  .regexpr_proc ()
	FUNCADDR  .open_proc (XLONG, XLONG, SLONG)
	FUNCADDR  .close_proc (XLONG, XLONG, XLONG)
	FUNCADDR  .pagecount_proc (XLONG, XLONG, XLONG)
	FUNCADDR  .pagecapability_proc (XLONG, XLONG, XLONG)
	FUNCADDR  .load_proc (XLONG, XLONG, SLONG, SLONG, XLONG)
	FUNCADDR  .save_proc (XLONG, XLONG, XLONG, SLONG, SLONG, XLONG)
	FUNCADDR  .validate_proc (XLONG, XLONG)
	FUNCADDR  .mime_proc ()
	FUNCADDR  .supports_export_bpp_proc (SLONG)
	FUNCADDR  .supports_export_type_proc (XLONG)
	FUNCADDR  .supports_icc_profiles_proc ()
END TYPE


' Load / Save flag constants -----------------------------------------------

$$BMP_DEFAULT         = 0
$$BMP_SAVE_RLE        = 1
$$CUT_DEFAULT         = 0
$$DDS_DEFAULT         = 0
$$GIF_DEFAULT         = 0
$$ICO_DEFAULT         = 0
$$ICO_MAKEALPHA       = 1   ' convert to 32bpp and create an alpha channel from the AND-mask when loading
$$IFF_DEFAULT         = 0
$$JPEG_DEFAULT        = 0
$$JPEG_FAST           = 1
$$JPEG_ACCURATE       = 2
$$JPEG_QUALITYSUPERB  = 0x80
$$JPEG_QUALITYGOOD    = 0x100
$$JPEG_QUALITYNORMAL  = 0x200
$$JPEG_QUALITYAVERAGE = 0x400
$$JPEG_QUALITYBAD     = 0x800
$$JPEG_CMYK           = 0x1000  ' load separated CMYK "as is"  (use | to combine with other flags)
$$KOALA_DEFAULT       = 0
$$LBM_DEFAULT         = 0
$$MNG_DEFAULT         = 0
$$PCD_DEFAULT         = 0
$$PCD_BASE            = 1   ' load the bitmap sized 768 x 512
$$PCD_BASEDIV4        = 2   ' load the bitmap sized 384 x 256
$$PCD_BASEDIV16       = 3   ' load the bitmap sized 192 x 128
$$PCX_DEFAULT         = 0
$$PNG_DEFAULT         = 0
$$PNG_IGNOREGAMMA     = 1   ' avoid gamma correction
$$PNM_DEFAULT         = 0
$$PNM_SAVE_RAW        = 0       ' If set the writer saves in RAW format  (i.e. P4, P5 or P6)
$$PNM_SAVE_ASCII      = 1       ' If set the writer saves in ASCII format  (i.e. P1, P2 or P3)
$$PSD_DEFAULT         = 0
$$RAS_DEFAULT         = 0
$$TARGA_DEFAULT       = 0
$$TARGA_LOAD_RGB888   = 1       ' If set the loader converts RGB555 and ARGB8888 -> RGB888.
$$TIFF_DEFAULT        = 0
$$TIFF_CMYK           = 0x0001  ' reads/stores tags for separated CMYK  (use | to combine with compression flags)
$$TIFF_PACKBITS       = 0x0100  ' save using PACKBITS compression
$$TIFF_DEFLATE        = 0x0200  ' save using DEFLATE compression  (a.k.a. ZLIB compression)
$$TIFF_ADOBE_DEFLATE  = 0x0400  ' save using ADOBE DEFLATE compression
$$TIFF_NONE           = 0x0800  ' save without any compression
$$TIFF_CCITTFAX3      = 0x1000  ' save using CCITT Group 3 fax encoding
$$TIFF_CCITTFAX4      = 0x2000  ' save using CCITT Group 4 fax encoding
$$TIFF_LZW            = 0x4000  ' save using LZW compression
$$TIFF_JPEG           = 0x8000  ' save using JPEG compression
$$WBMP_DEFAULT        = 0
$$XBM_DEFAULT         = 0
$$XPM_DEFAULT         = 0
END EXPORT


EXPORT
DECLARE FUNCTION  Xfree_Init ()
DECLARE FUNCTION  FreeImage_Initialise (XLONG)
DECLARE FUNCTION  FreeImage_DeInitialise ()
DECLARE FUNCTION  STRING FreeImage_GetVersion ()
DECLARE FUNCTION  STRING FreeImage_GetCopyrightMessage ()
DECLARE FUNCTION  FreeImage_SetOutputMessage (XLONG)
DECLARE FUNCTION  FreeImage_Allocate (XLONG, XLONG, XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_AllocateT (XLONG, XLONG, XLONG, XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_Clone (XLONG)
DECLARE FUNCTION  FreeImage_Unload (XLONG)
DECLARE FUNCTION  FreeImage_Load (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_LoadFromHandle (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_Save (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_SaveToHandle (XLONG, XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_OpenMemory (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_CloseMemory (XLONG)
DECLARE FUNCTION  FreeImage_LoadFromMemory (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_SaveToMemory (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_TellMemory (XLONG)
DECLARE FUNCTION  FreeImage_SeekMemory (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_AcquireMemory (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_RegisterLocalPlugin (XLONG, XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_RegisterExternalPlugin (XLONG, XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_GetFIFCount ()
DECLARE FUNCTION  FreeImage_SetPluginEnabled (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_IsPluginEnabled (XLONG)
DECLARE FUNCTION  FreeImage_GetFIFFromFormat (XLONG)
DECLARE FUNCTION  FreeImage_GetFIFFromMime (XLONG)
DECLARE FUNCTION  STRING FreeImage_GetFormatFromFIF (XLONG)
DECLARE FUNCTION  STRING FreeImage_GetFIFExtensionList (XLONG)
DECLARE FUNCTION  STRING FreeImage_GetFIFDescription (XLONG)
DECLARE FUNCTION  STRING FreeImage_GetFIFRegExpr (XLONG)
DECLARE FUNCTION  STRING FreeImage_GetFIFMimeType (XLONG)
DECLARE FUNCTION  FreeImage_GetFIFFromFilename (XLONG)
DECLARE FUNCTION  FreeImage_FIFSupportsReading (XLONG)
DECLARE FUNCTION  FreeImage_FIFSupportsWriting (XLONG)
DECLARE FUNCTION  FreeImage_FIFSupportsExportBPP (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_FIFSupportsExportType (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_FIFSupportsICCProfiles (XLONG)
DECLARE FUNCTION  FreeImage_OpenMultiBitmap (XLONG, XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_CloseMultiBitmap (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_GetPageCount (XLONG)
DECLARE FUNCTION  FreeImage_AppendPage (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_InsertPage (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_DeletePage (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_LockPage (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_UnlockPage (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_MovePage (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_GetLockedPageNumbers (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_GetFileType (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_GetFileTypeFromHandle (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_GetFileTypeFromMemory (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_GetImageType (XLONG)
DECLARE FUNCTION  FreeImage_IsLittleEndian ()
DECLARE FUNCTION  FreeImage_LookupX11Color (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_LookupSVGColor (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_GetBits (XLONG)
DECLARE FUNCTION  FreeImage_GetScanLine (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_GetPixelIndex (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_GetPixelColor (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_SetPixelIndex (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_SetPixelColor (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_GetColorsUsed (XLONG)
DECLARE FUNCTION  FreeImage_GetBPP (XLONG)
DECLARE FUNCTION  FreeImage_GetWidth (XLONG)
DECLARE FUNCTION  FreeImage_GetHeight (XLONG)
DECLARE FUNCTION  FreeImage_GetLine (XLONG)
DECLARE FUNCTION  FreeImage_GetPitch (XLONG)
DECLARE FUNCTION  FreeImage_GetDIBSize (XLONG)
DECLARE FUNCTION  RGBQUAD FreeImage_GetPalette (XLONG)
DECLARE FUNCTION  FreeImage_GetDotsPerMeterX (XLONG)
DECLARE FUNCTION  FreeImage_GetDotsPerMeterY (XLONG)
DECLARE FUNCTION  FreeImage_SetDotsPerMeterX (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_SetDotsPerMeterY (XLONG, XLONG)
DECLARE FUNCTION  BITMAPINFOHEADER FreeImage_GetInfoHeader (XLONG)
DECLARE FUNCTION  FreeImage_GetInfo (XLONG)
DECLARE FUNCTION  FreeImage_GetColorType (XLONG)
DECLARE FUNCTION  FreeImage_GetRedMask (XLONG)
DECLARE FUNCTION  FreeImage_GetGreenMask (XLONG)
DECLARE FUNCTION  FreeImage_GetBlueMask (XLONG)
DECLARE FUNCTION  FreeImage_GetTransparencyCount (XLONG)
DECLARE FUNCTION  FreeImage_GetTransparencyTable (XLONG)
DECLARE FUNCTION  FreeImage_SetTransparent (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_SetTransparencyTable (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_IsTransparent (XLONG)
DECLARE FUNCTION  FreeImage_HasBackgroundColor (XLONG)
DECLARE FUNCTION  FreeImage_GetBackgroundColor (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_SetBackgroundColor (XLONG, XLONG)
DECLARE FUNCTION  FIICCPROFILE FreeImage_GetICCProfile (XLONG)
DECLARE FUNCTION  FIICCPROFILE FreeImage_CreateICCProfile (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_DestroyICCProfile (XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine1To4 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine8To4 (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine16To4_555 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine16To4_565 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine24To4 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine32To4 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine1To8 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine4To8 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine16To8_555 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine16To8_565 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine24To8 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine32To8 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine1To16_555 (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine4To16_555 (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine8To16_555 (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine16_565_To16_555 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine24To16_555 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine32To16_555 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine1To16_565 (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine4To16_565 (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine8To16_565 (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine16_555_To16_565 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine24To16_565 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine32To16_565 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine1To24 (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine4To24 (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine8To24 (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine16To24_555 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine16To24_565 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine32To24 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine1To32 (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine4To32 (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine8To32 (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine16To32_555 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine16To32_565 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertLine24To32 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertTo4Bits (XLONG)
DECLARE FUNCTION  FreeImage_ConvertTo8Bits (XLONG)
DECLARE FUNCTION  FreeImage_ConvertTo16Bits555 (XLONG)
DECLARE FUNCTION  FreeImage_ConvertTo16Bits565 (XLONG)
DECLARE FUNCTION  FreeImage_ConvertTo24Bits (XLONG)
DECLARE FUNCTION  FreeImage_ConvertTo32Bits (XLONG)
DECLARE FUNCTION  FreeImage_ColorQuantize (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_Threshold (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_Dither (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertFromRawBits (XLONG, XLONG, XLONG, XLONG, XLONG, XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertToRawBits (XLONG, XLONG, XLONG, XLONG, XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertToStandardType (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ConvertToType (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ZLibCompress (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ZLibUncompress (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ZLibGZip (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_ZLibCRC32 (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_CreateTag ()
DECLARE FUNCTION  FreeImage_DeleteTag (XLONG)
DECLARE FUNCTION  FreeImage_CloneTag (XLONG)
DECLARE FUNCTION  STRING FreeImage_GetTagKey (XLONG)
DECLARE FUNCTION  STRING FreeImage_GetTagDescription (XLONG)
DECLARE FUNCTION  FreeImage_GetTagID (XLONG)
DECLARE FUNCTION  FreeImage_GetTagType (XLONG)
DECLARE FUNCTION  FreeImage_GetTagCount (XLONG)
DECLARE FUNCTION  FreeImage_GetTagLength (XLONG)
DECLARE FUNCTION  FreeImage_GetTagValue (XLONG)
DECLARE FUNCTION  FreeImage_SetTagKey (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_SetTagDescription (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_SetTagID (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_SetTagType (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_SetTagCount (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_SetTagLength (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_SetTagValue (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_FindFirstMetadata (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_FindNextMetadata (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_FindCloseMetadata (XLONG)
DECLARE FUNCTION  FreeImage_SetMetadata (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_GetMetadata (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_GetMetadataCount (XLONG, XLONG)
DECLARE FUNCTION  STRING FreeImage_TagToString (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_RotateClassic (XLONG, DOUBLE)
DECLARE FUNCTION  FreeImage_RotateEx (XLONG, DOUBLE, DOUBLE, DOUBLE, DOUBLE, DOUBLE, XLONG)
DECLARE FUNCTION  FreeImage_FlipHorizontal (XLONG)
DECLARE FUNCTION  FreeImage_FlipVertical (XLONG)
DECLARE FUNCTION  FreeImage_Rescale (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_AdjustCurve (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_AdjustGamma (XLONG, DOUBLE)
DECLARE FUNCTION  FreeImage_AdjustBrightness (XLONG, DOUBLE)
DECLARE FUNCTION  FreeImage_AdjustContrast (XLONG, DOUBLE)
DECLARE FUNCTION  FreeImage_Invert (XLONG)
DECLARE FUNCTION  FreeImage_GetHistogram (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_GetChannel (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_SetChannel (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_GetComplexChannel (XLONG, XLONG)
DECLARE FUNCTION  FreeImage_SetComplexChannel (XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_Copy (XLONG, XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_Paste (XLONG, XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  FreeImage_Composite (XLONG, XLONG, XLONG, XLONG)
DECLARE FUNCTION  Xfree_GetInfo (v0, BITMAPINFOHEADER bmih, RGBQUAD colorTable[])
END EXPORT


' ###########################
' #####  Xfree_Init ()  #####
' ###########################
'
' initialization function, called automatically
' when xfree.dll is loaded.
'
FUNCTION  Xfree_Init ()
	SHARED hlib
	hlib = LoadLibraryA (&"freeimage.dll")
END FUNCTION

FUNCTION FreeImage_Initialise (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_Initialise@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_DeInitialise ()
	SHARED hlib
	STATIC FUNCADDR f()
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_DeInitialise@0")
	RETURN @f()
END FUNCTION

FUNCTION STRING FreeImage_GetVersion ()
	SHARED hlib
	STATIC FUNCADDR f()
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetVersion@0")
	ret = @f()
	IF ret THEN	RETURN CSTRING$(ret) ELSE RETURN ""
END FUNCTION

FUNCTION STRING FreeImage_GetCopyrightMessage ()
	SHARED hlib
	STATIC FUNCADDR f()
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetCopyrightMessage@0")
	ret = @f()
	IF ret THEN	RETURN CSTRING$(ret) ELSE RETURN ""
END FUNCTION

FUNCTION FreeImage_SetOutputMessage (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetOutputMessage@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_Allocate (v0, v1, v2, v3, v4, v5)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_Allocate@24")
	RETURN @f(v0, v1, v2, v3, v4, v5)
END FUNCTION

FUNCTION FreeImage_AllocateT (v0, v1, v2, v3, v4, v5, v6)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_AllocateT@28")
	RETURN @f(v0, v1, v2, v3, v4, v5, v6)
END FUNCTION

FUNCTION FreeImage_Clone (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_Clone@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_Unload (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_Unload@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_Load (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_Load@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_LoadFromHandle (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_LoadFromHandle@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_Save (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_Save@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_SaveToHandle (v0, v1, v2, v3, v4)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SaveToHandle@20")
	RETURN @f(v0, v1, v2, v3, v4)
END FUNCTION

FUNCTION FreeImage_OpenMemory (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_OpenMemory@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_CloseMemory (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_CloseMemory@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_LoadFromMemory (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_LoadFromMemory@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_SaveToMemory (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SaveToMemory@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_TellMemory (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_TellMemory@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_SeekMemory (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SeekMemory@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_AcquireMemory (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_AcquireMemory@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_RegisterLocalPlugin (v0, v1, v2, v3, v4)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_RegisterLocalPlugin@20")
	RETURN @f(v0, v1, v2, v3, v4)
END FUNCTION

FUNCTION FreeImage_RegisterExternalPlugin (v0, v1, v2, v3, v4)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_RegisterExternalPlugin@20")
	RETURN @f(v0, v1, v2, v3, v4)
END FUNCTION

FUNCTION FreeImage_GetFIFCount ()
	SHARED hlib
	STATIC FUNCADDR f()
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetFIFCount@0")
	RETURN @f()
END FUNCTION

FUNCTION FreeImage_SetPluginEnabled (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetPluginEnabled@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_IsPluginEnabled (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_IsPluginEnabled@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetFIFFromFormat (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetFIFFromFormat@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetFIFFromMime (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetFIFFromMime@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION STRING FreeImage_GetFormatFromFIF (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetFormatFromFIF@4")
	ret = @f(v0)
	IF ret THEN	RETURN CSTRING$(ret) ELSE RETURN ""
END FUNCTION

FUNCTION STRING FreeImage_GetFIFExtensionList (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetFIFExtensionList@4")
	ret = @f(v0)
	IF ret THEN	RETURN CSTRING$(ret) ELSE RETURN ""
END FUNCTION

FUNCTION STRING FreeImage_GetFIFDescription (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetFIFDescription@4")
	ret = @f(v0)
	IF ret THEN	RETURN CSTRING$(ret) ELSE RETURN ""
END FUNCTION

FUNCTION STRING FreeImage_GetFIFRegExpr (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetFIFRegExpr@4")
	ret = @f(v0)
	IF ret THEN	RETURN CSTRING$(ret) ELSE RETURN ""
END FUNCTION

FUNCTION STRING FreeImage_GetFIFMimeType (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetFIFMimeType@4")
	ret = @f(v0)
	IF ret THEN	RETURN CSTRING$(ret) ELSE RETURN ""
END FUNCTION

FUNCTION FreeImage_GetFIFFromFilename (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetFIFFromFilename@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_FIFSupportsReading (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_FIFSupportsReading@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_FIFSupportsWriting (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_FIFSupportsWriting@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_FIFSupportsExportBPP (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_FIFSupportsExportBPP@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_FIFSupportsExportType (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_FIFSupportsExportType@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_FIFSupportsICCProfiles (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_FIFSupportsICCProfiles@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_OpenMultiBitmap (v0, v1, v2, v3, v4)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_OpenMultiBitmap@20")
	RETURN @f(v0, v1, v2, v3, v4)
END FUNCTION

FUNCTION FreeImage_CloseMultiBitmap (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_CloseMultiBitmap@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_GetPageCount (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetPageCount@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_AppendPage (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_AppendPage@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_InsertPage (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_InsertPage@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_DeletePage (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_DeletePage@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_LockPage (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_LockPage@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_UnlockPage (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_UnlockPage@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_MovePage (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_MovePage@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_GetLockedPageNumbers (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetLockedPageNumbers@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_GetFileType (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetFileType@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_GetFileTypeFromHandle (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetFileTypeFromHandle@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_GetFileTypeFromMemory (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetFileTypeFromMemory@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_GetImageType (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetImageType@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_IsLittleEndian ()
	SHARED hlib
	STATIC FUNCADDR f()
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_IsLittleEndian@0")
	RETURN @f()
END FUNCTION

FUNCTION FreeImage_LookupX11Color (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_LookupX11Color@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_LookupSVGColor (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_LookupSVGColor@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_GetBits (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetBits@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetScanLine (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetScanLine@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_GetPixelIndex (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetPixelIndex@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_GetPixelColor (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetPixelColor@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_SetPixelIndex (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetPixelIndex@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_SetPixelColor (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetPixelColor@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_GetColorsUsed (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetColorsUsed@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetBPP (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetBPP@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetWidth (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetWidth@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetHeight (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetHeight@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetLine (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetLine@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetPitch (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetPitch@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetDIBSize (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetDIBSize@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION RGBQUAD FreeImage_GetPalette (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	RGBQUAD rgb
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetPalette@4")
	ret = @f(v0)
	IF ret THEN RtlMoveMemory (&rbg, ret, SIZE(RGBQUAD))
	RETURN rgb
END FUNCTION

FUNCTION FreeImage_GetDotsPerMeterX (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetDotsPerMeterX@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetDotsPerMeterY (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetDotsPerMeterY@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_SetDotsPerMeterX (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetDotsPerMeterX@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_SetDotsPerMeterY (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetDotsPerMeterY@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION BITMAPINFOHEADER FreeImage_GetInfoHeader (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	BITMAPINFOHEADER bmih
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetInfoHeader@4")
	ret = @f(v0)
	IF ret THEN RtlMoveMemory (&bmih, ret, SIZE(BITMAPINFOHEADER))
	RETURN bmih
END FUNCTION

' See Xfree_GetInfo() for an alternative wrapper for this function.
FUNCTION FreeImage_GetInfo (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetInfo@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetColorType (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetColorType@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetRedMask (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetRedMask@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetGreenMask (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetGreenMask@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetBlueMask (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetBlueMask@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetTransparencyCount (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetTransparencyCount@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetTransparencyTable (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetTransparencyTable@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_SetTransparent (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetTransparent@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_SetTransparencyTable (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetTransparencyTable@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_IsTransparent (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_IsTransparent@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_HasBackgroundColor (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_HasBackgroundColor@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetBackgroundColor (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetBackgroundColor@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_SetBackgroundColor (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetBackgroundColor@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FIICCPROFILE FreeImage_GetICCProfile (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	FIICCPROFILE icc
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetICCProfile@4")
	ret = @f(v0)
	IF ret THEN RtlMoveMemory (&icc, ret, SIZE(FIICCPROFILE))
	RETURN icc
END FUNCTION

FUNCTION FIICCPROFILE FreeImage_CreateICCProfile (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	FIICCPROFILE icc
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_CreateICCProfile@12")
	ret = @f(v0, v1, v2)
	IF ret THEN RtlMoveMemory (&icc, ret, SIZE(FIICCPROFILE))
	RETURN icc
END FUNCTION

FUNCTION FreeImage_DestroyICCProfile (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_DestroyICCProfile@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_ConvertLine1To4 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine1To4@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine8To4 (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine8To4@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_ConvertLine16To4_555 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine16To4_555@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine16To4_565 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine16To4_565@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine24To4 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine24To4@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine32To4 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine32To4@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine1To8 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine1To8@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine4To8 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine4To8@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine16To8_555 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine16To8_555@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine16To8_565 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine16To8_565@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine24To8 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine24To8@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine32To8 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine32To8@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine1To16_555 (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine1To16_555@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_ConvertLine4To16_555 (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine4To16_555@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_ConvertLine8To16_555 (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine8To16_555@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_ConvertLine16_565_To16_555 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine16_565_To16_555@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine24To16_555 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine24To16_555@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine32To16_555 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine32To16_555@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine1To16_565 (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine1To16_565@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_ConvertLine4To16_565 (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine4To16_565@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_ConvertLine8To16_565 (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine8To16_565@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_ConvertLine16_555_To16_565 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine16_555_To16_565@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine24To16_565 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine24To16_565@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine32To16_565 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine32To16_565@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine1To24 (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine1To24@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_ConvertLine4To24 (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine4To24@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_ConvertLine8To24 (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine8To24@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_ConvertLine16To24_555 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine16To24_555@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine16To24_565 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine16To24_565@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine32To24 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine32To24@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine1To32 (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine1To32@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_ConvertLine4To32 (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine4To32@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_ConvertLine8To32 (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine8To32@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_ConvertLine16To32_555 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine16To32_555@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine16To32_565 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine16To32_565@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertLine24To32 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertLine24To32@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ConvertTo4Bits (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertTo4Bits@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_ConvertTo8Bits (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertTo8Bits@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_ConvertTo16Bits555 (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertTo16Bits555@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_ConvertTo16Bits565 (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertTo16Bits565@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_ConvertTo24Bits (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertTo24Bits@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_ConvertTo32Bits (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertTo32Bits@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_ColorQuantize (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ColorQuantize@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_Threshold (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_Threshold@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_Dither (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_Dither@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_ConvertFromRawBits (v0, v1, v2, v3, v4, v5, v6, v7, v8)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG, XLONG, XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertFromRawBits@36")
	RETURN @f(v0, v1, v2, v3, v4, v5, v6, v7, v8)
END FUNCTION

FUNCTION FreeImage_ConvertToRawBits (v0, v1, v2, v3, v4, v5, v6, v7)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG, XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertToRawBits@32")
	RETURN @f(v0, v1, v2, v3, v4, v5, v6, v7)
END FUNCTION

FUNCTION FreeImage_ConvertToStandardType (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertToStandardType@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_ConvertToType (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ConvertToType@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_ZLibCompress (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ZLibCompress@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_ZLibUncompress (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ZLibUncompress@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_ZLibGZip (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ZLibGZip@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_ZLibCRC32 (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_ZLibCRC32@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_CreateTag ()
	SHARED hlib
	STATIC FUNCADDR f()
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_CreateTag@0")
	RETURN @f()
END FUNCTION

FUNCTION FreeImage_DeleteTag (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_DeleteTag@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_CloneTag (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_CloneTag@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION STRING FreeImage_GetTagKey (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetTagKey@4")
	ret = @f(v0)
	IF ret THEN RETURN CSTRING$(ret) ELSE RETURN ""
END FUNCTION

FUNCTION STRING FreeImage_GetTagDescription (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetTagDescription@4")
	ret = @f(v0)
	IF ret THEN RETURN CSTRING$(ret) ELSE RETURN ""
END FUNCTION

FUNCTION FreeImage_GetTagID (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetTagID@4")
	ret = @f(v0) & 0xFFFF 'return value is a USHORT, make sure high bytes are clear
	RETURN ret
END FUNCTION

FUNCTION FreeImage_GetTagType (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetTagType@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetTagCount (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetTagCount@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetTagLength (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetTagLength@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetTagValue (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetTagValue@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_SetTagKey (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetTagKey@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_SetTagDescription (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetTagDescription@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_SetTagID (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetTagID@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_SetTagType (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetTagType@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_SetTagCount (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetTagCount@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_SetTagLength (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetTagLength@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_SetTagValue (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetTagValue@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_FindFirstMetadata (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_FindFirstMetadata@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_FindNextMetadata (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_FindNextMetadata@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_FindCloseMetadata (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_FindCloseMetadata@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_SetMetadata (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetMetadata@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_GetMetadata (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetMetadata@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_GetMetadataCount (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetMetadataCount@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION STRING FreeImage_TagToString (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_TagToString@12")
	ret = @f(v0, v1, v2)
	IF ret THEN RETURN CSTRING$(ret) ELSE RETURN ""
END FUNCTION

FUNCTION FreeImage_RotateClassic (v0, v1#)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, DOUBLE)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_RotateClassic@12")
	RETURN @f(v0, v1#)
END FUNCTION

FUNCTION FreeImage_RotateEx (v0, v1#, v2#, v3#, v4#, v5#, v6)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, DOUBLE, DOUBLE, DOUBLE, DOUBLE, DOUBLE, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_RotateEx@48")
	RETURN @f(v0, v1#, v2#, v3#, v4#, v5#, v6)
END FUNCTION

FUNCTION FreeImage_FlipHorizontal (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_FlipHorizontal@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_FlipVertical (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_FlipVertical@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_Rescale (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_Rescale@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

FUNCTION FreeImage_AdjustCurve (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_AdjustCurve@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_AdjustGamma (v0, v1#)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, DOUBLE)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_AdjustGamma@12")
	RETURN @f(v0, v1#)
END FUNCTION

FUNCTION FreeImage_AdjustBrightness (v0, v1#)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, DOUBLE)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_AdjustBrightness@12")
	RETURN @f(v0, v1#)
END FUNCTION

FUNCTION FreeImage_AdjustContrast (v0, v1#)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, DOUBLE)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_AdjustContrast@12")
	RETURN @f(v0, v1#)
END FUNCTION

FUNCTION FreeImage_Invert (v0)
	SHARED hlib
	STATIC FUNCADDR f(XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_Invert@4")
	RETURN @f(v0)
END FUNCTION

FUNCTION FreeImage_GetHistogram (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetHistogram@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_GetChannel (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetChannel@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_SetChannel (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetChannel@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_GetComplexChannel (v0, v1)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetComplexChannel@8")
	RETURN @f(v0, v1)
END FUNCTION

FUNCTION FreeImage_SetComplexChannel (v0, v1, v2)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_SetComplexChannel@12")
	RETURN @f(v0, v1, v2)
END FUNCTION

FUNCTION FreeImage_Copy (v0, v1, v2, v3, v4)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_Copy@20")
	RETURN @f(v0, v1, v2, v3, v4)
END FUNCTION

FUNCTION FreeImage_Paste (v0, v1, v2, v3, v4)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_Paste@20")
	RETURN @f(v0, v1, v2, v3, v4)
END FUNCTION

FUNCTION FreeImage_Composite (v0, v1, v2, v3)
	SHARED hlib
	STATIC FUNCADDR f(XLONG, XLONG, XLONG, XLONG)
	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_Composite@16")
	RETURN @f(v0, v1, v2, v3)
END FUNCTION

'The following function is provided as an alternative to FreeImage_GetInfo().
' The FreeImage function returns a pointer to a BITMAPINFO structure, which
' contains a variable-length colour table. XBasic cannot represent variable-
' length members in a composite type, so Xfree_GetInfo() returns the two
' members of the BITMAPINFO as separate arguments.
'
FUNCTION Xfree_GetInfo (v0, BITMAPINFOHEADER bmih, RGBQUAD colorTable[])
	SHARED hlib
	STATIC FUNCADDR f(XLONG)

	IFZ f THEN f = GetProcAddress (hlib, &"_FreeImage_GetInfo@4")
	bmi = @f(v0) 'pointer to BITMAPINFO

	IF bmi THEN
		RtlMoveMemory (&bmih, bmi, SIZE(BITMAPINFOHEADER))
		SELECT CASE bmih.biBitCount
			CASE 1: DIM colorTable[1]
			CASE 4: DIM colorTable[15]
			CASE 8: DIM colorTable[255]
			CASE ELSE: DIM colorTable[]
		END SELECT
		IF colorTable[] THEN
			table = bmi + SIZE(BITMAPINFOHEADER)
			RtlMoveMemory (&colorTable[], table, SIZE(colorTable[]))
		END IF
	END IF

	RETURN bmi

END FUNCTION
END PROGRAM
